@@ -34,21 +34,13 @@ except:
 
 # Doc tests should be run as well.
 def load_tests(loader, tests, ignore):
-    tests.addTests(doctest.DocTestSuite('glymur.jp2k'))
+    if glymur.lib.openjp2._OPENJP2 is not None:
+        tests.addTests(doctest.DocTestSuite('glymur.jp2k'))
     return tests
 
 
-@contextlib.contextmanager
-def chdir(dirname=None):
-    curdir = os.getcwd()
-    try:
-        if dirname is not None:
-            os.chdir(dirname)
-        yield
-    finally:
-        os.chdir(curdir)
-
-
+@unittest.skipIf(glymur.lib.openjp2._OPENJP2 is None,
+                 "Missing openjp2 library.")
 class TestJp2k(unittest.TestCase):
 
     @classmethod
@@ -607,13 +599,15 @@ class TestJp2k(unittest.TestCase):
     def test_config_file_via_environ(self):
         """Verify that we can read a configuration file set via environ var."""
         with tempfile.TemporaryDirectory() as tdir:
-            filename = os.path.join(tdir, 'glymurrc')
+            configdir = os.path.join(tdir, 'glymur')
+            os.mkdir(configdir)
+            filename = os.path.join(configdir, 'glymurrc')
             with open(filename, 'wb') as tfile:
                 tfile.write('[library]\n'.encode())
                 line = 'openjp2: {0}\n'.format(glymur._OPENJP2._name)
                 tfile.write(line.encode())
                 tfile.flush()
-                with patch.dict('os.environ', {'GLYMURCONFIGDIR': tdir}):
+                with patch.dict('os.environ', {'XDG_CONFIG_HOME': tdir}):
                     imp.reload(glymur)
                     j = Jp2k(self.jp2file)
 
@@ -622,13 +616,15 @@ class TestJp2k(unittest.TestCase):
     def test_config_file_via_environ_is_wrong(self):
         # A non-existant library location should be rejected.
         with tempfile.TemporaryDirectory() as tdir:
-            fname = os.path.join(tdir, 'glymurrc')
+            configdir = os.path.join(tdir, 'glymur')
+            os.mkdir(configdir)
+            fname = os.path.join(configdir, 'glymurrc')
             with open(fname, 'w') as fp:
                 with tempfile.NamedTemporaryFile(suffix='.dylib') as tfile:
                     fp.write('[library]\n')
                     fp.write('openjp2: {0}.not.there\n'.format(tfile.name))
                     fp.flush()
-                    with patch.dict('os.environ', {'GLYMURCONFIGDIR': tdir}):
+                    with patch.dict('os.environ', {'XDG_CONFIG_HOME': tdir}):
                         # Misconfigured new configuration file should
                         # be rejected.
                         with self.assertWarns(UserWarning) as cw:
@@ -640,7 +636,7 @@ class TestJp2k(unittest.TestCase):
         # Verify that we error out properly if the configuration file
         # specified via environment variable is not found.
         with tempfile.TemporaryDirectory() as tdir:
-            with patch.dict('os.environ', {'GLYMURCONFIGDIR': tdir}):
+            with patch.dict('os.environ', {'XDG_CONFIG_HOME': tdir}):
                 # Misconfigured new configuration file should
                 # be rejected.
                 with self.assertWarns(UserWarning) as cw:
@@ -649,7 +645,7 @@ class TestJp2k(unittest.TestCase):
     @unittest.skipIf(sys.hexversion < 0x03020000,
                      "Uses features introduced in 3.2.")
     def test_home_dir_missing_config_dir(self):
-        # Verify no exception is raised if $HOME is missing .glymur directory.
+        # Verify no exception is raised if $HOME is missing .config directory.
         with tempfile.TemporaryDirectory() as tdir:
             with patch.dict('os.environ', {'HOME': tdir}):
                 # Misconfigured new configuration file should
@@ -657,6 +653,24 @@ class TestJp2k(unittest.TestCase):
                 with self.assertWarns(UserWarning) as cw:
                     imp.reload(glymur)
 
+    @unittest.skipIf(sys.hexversion < 0x03020000,
+                     "Uses features introduced in 3.2.")
+    def test_home_dir_missing_glymur_rc_dir(self):
+        # Should warn but not error if $HOME/.config but no glymurrc dir.
+        with tempfile.TemporaryDirectory() as tdir:
+            # We need the subdirectory to be specifically named as ".config"
+            # in order for this test to work.  A specifically-named temporary
+            # directory does not seem to be possible, so try to symlink it.
+            # Supposedly the symlink gets cleaned up with tdir gets cleaned up.
+            with tempfile.TemporaryDirectory(suffix=".config", dir=tdir) \
+                    as tdir_config:
+                os.symlink(tdir_config, os.path.join(tdir, '.config'))
+                with patch.dict('os.environ', {'HOME': tdir}):
+                    # Misconfigured new configuration file should
+                    # be rejected.
+                    with self.assertWarns(UserWarning) as cw:
+                        imp.reload(glymur)
+
     def test_xmp_attribute(self):
         # Verify that we can read the XMP packet in our shipping example file.
         j = Jp2k(self.jp2file)
